# Setup script for system INI file
# Setup script is the place to define the migration procedure
#  
import os, sys, logging, shutil, time, zipfile
import setup_util

_SvcName = "MSSQL$SQLEXPRESS"
_ModName = "db"

''' Install to latest '''
def install():
 if setup_util.i4j["tm.db.external"]=="true" or setup_util.i4j["tm.upgrade.opt"]=="upgrade":
  logging.info("[setup_db.py] skipped for using external()")
  setup_util.set_progress(100)
  return
 setup_util.set_progress(10)
 logging.info("[setup_db.py] start install()")
 if not setup_util.sc_exists(_SvcName):
  logging.info("[setup_db.py] install embedded SQLExpress")
  tmpdist = setup_util.i4j["sys.installationDir"]+"\\db\\dist"
  logging.info("[setup_db.py] extract database setup to ... "+tmpdist)
  zipfile.ZipFile(setup_util.i4j["sys.installationDir"]+"\\db\\dist\\dist.zip","r").extractall(tmpdist)
  logging.info("[setup_db.py] install database ...")
  setup_util.set_progress(15)
  os.chdir(tmpdist+"\\dist")
  sqlxOpts = "/IAcceptSQLServerLicenseTerms \
   /Features=SQL \
   /InstanceName=SQLExpress /SQLSVCACCOUNT=\"NT AUTHORITY\\SYSTEM\" \
   /SQLSYSADMINACCOUNTS=\"Builtin\\Administrators\" /TCPENABLED=1"
  setup_inst="Setup.exe /QS /Action=Install "
  if "UNATTENDED" in os.environ:
   setup_inst="Setup.exe /Q /Action=Install "
  ret, out=setup_util.psys(setup_inst+sqlxOpts)
  if not 0==ret:
   raise Exception("Failed to install SQL express:"+out)
 setup_util.set_progress(80)
 logging.info("[setup_db.py] Creating application database...")
 sqlcmd = "E\:\\Program Files\\Microsoft SQL Server\\100\\Tools\\Binn\\SQLCMD.EXE"
 createdb = [sqlcmd, "-S", "localhost\\SQLEXPRESS", "-Q", "CREATE DATABASE "+setup_util.i4j["tm.db.name"]+";"]
 logging.info("[setup_db.py] [SQLCMD] "+str(createdb))
 ret, out=setup_util.psys(createdb)
 maxtry = 10
 while len(out)>0 and out.startswith("Msg ") and maxtry>0:
  maxtry = maxtry-1
  if out.startswith("Msg 5170"):
   dbf = out.split("'")[1]
   os.remove(dbf)
   ret, out=setup_util.psys(createdb)
  elif out.startswith("Msg 1801"):
   dropdb = [sqlcmd, "-S", "localhost\\SQLEXPRESS", "-Q", "DROP DATABASE "+setup_util.i4j["tm.db.name"]+";"]
   setup_util.psys(dropdb)
   logging.info("[setup_db.py] drop existing db and recreate it")
   ret, out=setup_util.psys(createdb)
  else:
   logging.error(out)
   raise Exception("Failed to create database:"+out)
 #leave a flag file to indicat builtin sql
 with open(setup_util.i4j["sys.installationDir"]+"\\db\\dist\\builtin.dat", "wb") as tf:
  tf.write("1")
 tf.close()   
 #cleanup
 setup_util.safe_remove(setup_util.i4j["sys.installationDir"]+"\\db\\dist\\dist")
 setup_util.safe_remove(setup_util.i4j["sys.installationDir"]+"\\db\\dist\\dist.zip")
 setup_util.sc_start(_SvcName)

''' Maintain for upgrade ''' 
def maintain():
 pass

''' Completely uninstall component '''
def uninstall():
 maintain()
 if os.path.exists(setup_util.i4j["sys.installationDir"]+"\\db\\dist\\builtin.dat") and setup_util.sc_exists(_SvcName):
  logging.info("[setup_db.py] cleanup SQL express")
  #stop and delete database
  sqlxOpts = "/IAcceptSQLServerLicenseTerms \
   /Features=SQL \
   /InstanceName=SQLExpress "
  setup_uninst="Setup.exe /QS /Action=Uninstall "
  if "UNATTENDED" in os.environ:
   setup_uninst="Setup.exe /Q /Action=Uninstall "
   path = "E:\Program Files\Microsoft SQL Server\100\Setup Bootstrap\SQLServer2008R2"
  os.chdir (path)
  logging.info("[setup_db.py] uninstall SQL express")
  ret, out=setup_util.psys(setup_uninst+sqlxOpts)
  if not ret==0:
   logging.error("Failed to uninstall SQL express:"+out)

''' Standard setup script main function
	 * The python installer script must implement the following methods
	 * -i install
	 * -p upgrade based on version spec
	 * -m maintain for upgrade
	 * -x complete uninstall
'''
def main():
 setup_util.init()
 try:
  if sys.argv[1]=="-i":
   install()
  elif sys.argv[1]=="-p":
   migrate_call = getattr(sys.modules[__name__],"migrate_"+setup_util.i4j["tm.migrate.v"],None)
   if not None==migrate_call:
    migrate_call()
  elif sys.argv[1]=="-m":
   maintain()
  elif sys.argv[1]=="-x":
   uninstall()
  setup_util.set_progress(100)  
  return 0
 except Exception, x:
  logging.exception(x)
  setup_util.set_progress(999)  
  return -1 

if __name__=="__main__":
 sys.exit(main())
